#include "stdafx.h"
#include "PHue++.h"


constexpr int LIGHTID_TO_USE{1}; //Please change this value to suit your own Hue light setup
const wchar_t* BRIDGE_ADDRESS{L"172.16.18.132"}; //Please change this IP address value to suit your own Hue light setup


int main()
{
  //Initialise Sockets
  WSADATA wsaData{};
  if (WSAStartup(MAKEWORD(2, 2), &wsaData) != 0)
  {
    const DWORD dwLastError{GetLastError()};
    _tprintf(_T("Failed to initialise Winsock, Error:%u\n"), dwLastError); //NOLINT(clang-diagnostic-format)
    return dwLastError;
  }

  //Create the WinHTTP session object
  WinHTTPWrappers::CSession session;
#pragma warning(suppress: 26477)
  HRESULT hr{session.Initialize(L"PHue++", WINHTTP_ACCESS_TYPE_DEFAULT_PROXY, WINHTTP_NO_PROXY_NAME, WINHTTP_NO_PROXY_BYPASS, 0)};
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));

  //Try out the temperature conversion functions
  int32_t mired{PhilipsHue::KelvinToMired(2000)}; //NOLINT(clang-diagnostic-unused-but-set-variable)
  mired = PhilipsHue::KelvinToMired(6500); //NOLINT(clang-analyzer-deadcode.DeadStores)
  int32_t kelvin{PhilipsHue::MiredToKelvin(500)}; //NOLINT(clang-diagnostic-unused-but-set-variable)
  kelvin = PhilipsHue::MiredToKelvin(153); //NOLINT(clang-analyzer-deadcode.DeadStores)

  //Try out the color space conversion functions
  std::pair<float, float> xy{PhilipsHue::RGBToXY(RGB(255, 0, 0))};
  xy = PhilipsHue::RGBToXY(RGB(255, 0, 0), PhilipsHue::g_GamutA);
  printf("GamutA, Red   -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  COLORREF c{PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutA)};
  printf("GamutA, (%f, %f) -> Red (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 255, 0), PhilipsHue::g_GamutA);
  printf("GamutA, Green -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutA);
  printf("GamutA, (%f, %f) -> Green (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 0, 255), PhilipsHue::g_GamutA);
  printf("GamutA, Blue  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutA);
  printf("GamutA, (%f, %f) -> Blue (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(255, 255, 255), PhilipsHue::g_GamutA);
  printf("GamutA, White  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutA);
  printf("GamutA, (%f, %f) -> White (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(255, 0, 0), PhilipsHue::g_GamutB);
  printf("GamutB, Red   -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutB);
  printf("GamutB, (%f, %f) -> Red (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 255, 0), PhilipsHue::g_GamutB);
  printf("GamutB, Green -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutB);
  printf("GamutB, (%f, %f) -> Green (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 0, 255), PhilipsHue::g_GamutB);
  printf("GamutB, Blue  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutB);
  printf("GamutB, (%f, %f) -> Blue (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(255, 255, 255), PhilipsHue::g_GamutB);
  printf("GamutB, White  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutB);
  printf("GamutB, (%f, %f) -> White (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(255, 0, 0), PhilipsHue::g_GamutC);
  printf("GamutC, Red   -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutC);
  printf("GamutC, (%f, %f) -> Red (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 255, 0), PhilipsHue::g_GamutC);
  printf("GamutC, Green -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutC);
  printf("GamutC, (%f, %f) -> Green (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 0, 255), PhilipsHue::g_GamutC);
  printf("GamutC, Blue  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutC);
  printf("GamutC, (%f, %f) -> Blue (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(255, 255, 255), PhilipsHue::g_GamutC);
  printf("GamutC, White  -> (%f, %f)\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  c = PhilipsHue::XYToRGB(xy.first, xy.second, 1, PhilipsHue::g_GamutC);
  printf("GamutC, (%f, %f) -> White (%d, %d, %d)\n", static_cast<double>(xy.first), static_cast<double>(xy.second), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  xy = PhilipsHue::RGBToXY(RGB(0, 255, 0));
  xy = PhilipsHue::RGBToXY(RGB(0, 0, 255));
  xy = PhilipsHue::RGBToXY(RGB(255, 255, 255));
  c = PhilipsHue::XYToRGB(0.675f, 0.322f, 0.5f); //NOLINT(clang-analyzer-deadcode.DeadStores)
  c = PhilipsHue::XYToRGB(0.73f, 0.25f, 0.5f); //NOLINT(clang-analyzer-deadcode.DeadStores)
  c = PhilipsHue::XYToRGB(0.1f, 0.83f, 0.5f); //NOLINT(clang-analyzer-deadcode.DeadStores)
  c = PhilipsHue::XYToRGB(0.15f, 0.05f, 0.5f); //NOLINT(clang-analyzer-deadcode.DeadStores)

  std::array<float, 3> hsl{PhilipsHue::RGBToHSL(RGB(255, 0, 0))};
#pragma warning(suppress: 26446)
  printf("Red -> HSL (%f, %f, %f)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]));
#pragma warning(suppress: 26446)
  c = PhilipsHue::HSLToRGB(hsl[0], hsl[1], hsl[2]);
#pragma warning(suppress: 26446)
  printf("(%f, %f, %f) -> Red (%d, %d, %d)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  hsl = PhilipsHue::RGBToHSL(RGB(0, 255, 0));
#pragma warning(suppress: 26446)
  printf("Green -> HSL (%f, %f, %f)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]));
#pragma warning(suppress: 26446)
  c = PhilipsHue::HSLToRGB(hsl[0], hsl[1], hsl[2]);
#pragma warning(suppress: 26446)
  printf("(%f, %f, %f) -> Green (%d, %d, %d)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  hsl = PhilipsHue::RGBToHSL(RGB(0, 0, 255));
#pragma warning(suppress: 26446)
  printf("Blue -> HSL (%f, %f, %f)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]));
#pragma warning(suppress: 26446)
  c = PhilipsHue::HSLToRGB(hsl[0], hsl[1], hsl[2]);
#pragma warning(suppress: 26446)
  printf("(%f, %f, %f) -> Blue (%d, %d, %d)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  hsl = PhilipsHue::RGBToHSL(RGB(0, 0, 0));
#pragma warning(suppress: 26446)
  printf("Black -> HSL (%f, %f, %f)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]));
#pragma warning(suppress: 26446)
  c = PhilipsHue::HSLToRGB(hsl[0], hsl[1], hsl[2]);
#pragma warning(suppress: 26446)
  printf("(%f, %f, %f) -> Black (%d, %d, %d)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  hsl = PhilipsHue::RGBToHSL(RGB(255, 255, 255));
#pragma warning(suppress: 26446)
  printf("White -> HSL (%f, %f, %f)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]));
#pragma warning(suppress: 26446)
  c = PhilipsHue::HSLToRGB(hsl[0], hsl[1], hsl[2]);
#pragma warning(suppress: 26446)
  printf("(%f, %f, %f) -> White (%d, %d, %d)\n", static_cast<double>(hsl[0]), static_cast<double>(hsl[1]), static_cast<double>(hsl[2]), static_cast<int>(GetRValue(c)), static_cast<int>(GetGValue(c)), static_cast<int>(GetBValue(c)));

  //Try out the UPnP discovery function
  std::vector<std::pair<std::string, std::string>> devices;
  hr = PhilipsHue::UPnPDiscover(devices, 5, "ssdp:all", "PHue++"); //NOLINT(clang-analyzer-deadcode.DeadStores)
  printf("Found the following UPnP devices on the local network\n");
  for (const auto& device : devices)
#pragma warning(suppress: 26489)
    printf("LOCATION:%s, SERVER:%s\n", device.first.c_str(), device.second.c_str());

  //Create the bridge object
  PhilipsHue::CBridge bridge;
  hr = bridge.Initialize(session, BRIDGE_ADDRESS);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));

  //Exercise the various methods of the bridge object
  std::string sResponse;
  hr = bridge.GetDescriptionXML(sResponse);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  printf("GetDescriptionXML response was %s\n", sResponse.c_str());

  std::vector<PhilipsHue::CErrorResponse> errors;
  std::wstring sClientKey;
  hr = bridge.CreateNewUser("newdeveloper", sClientKey, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)

  hr = bridge.Get(L"/api/ehnWDv2pqM6-OzRU8tsGSjZFAo7UOfpop9xBnsV8/", sResponse);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  printf("Authorized username response was %s\n", sResponse.c_str());

  hr = bridge.Post(L"/api", R"({"devicetype":"my_hue_app#iphone peter"})", sResponse);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  bool bError{false};
  hr = bridge.IsErrorResponse(sResponse, bError, errors);
#pragma warning(suppress: 26472)
  printf("Create new username response was %s, hr: %08X, error response:%d\n", sResponse.c_str(), static_cast<unsigned int>(hr), static_cast<int>(bError));

#pragma warning(suppress: 26489)
  bridge.m_sClientKey = L"ehnWDv2pqM6-OzRU8tsGSjZFAo7UOfpop9xBnsV8";

  hr = bridge.GetFullState(errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hr = bridge.GetConfiguration(errors);  //NOLINT(clang-analyzer-deadcode.DeadStores)
  hr = bridge.DeleteUser("newdeveloper", "uU4VV7RfnztRUbW55b2tBDILjaAMxhxtjQ6jLD6-", errors); //NOLINT(clang-analyzer-deadcode.DeadStores)

  hr = bridge.GetLights(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  bool bHasLight{bridge.HasLight(42)}; //NOLINT(clang-analyzer-deadcode.DeadStores, clang-diagnostic-unused-but-set-variable)
  bHasLight = bridge.HasLight(1); //NOLINT(clang-analyzer-deadcode.DeadStores)

  /*
  std::vector<int> newLights;
  std::wstring sLastScan;
  hr = bridge.GetNewLights(newLights, sLastScan, errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  hr = bridge.SearchNewLights(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  Sleep(1000);
  hr = bridge.GetNewLights(newLights, sLastScan, errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  Sleep(40000);
  hr = bridge.GetNewLights(newLights, sLastScan, errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  */

  PhilipsHue::CLight light1;
#pragma warning(suppress: 26486)
  hr = bridge.GetLight(LIGHTID_TO_USE, light1);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));

  hr = light1.SetOff(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  hr = light1.SetOn(errors);

#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  hr = light1.SetOff(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  hr = light1.SetOn(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));

  PhilipsHue::CLightState state;
  state.m_Alert = PhilipsHue::CLightState::ALERT::SELECT;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Alert = PhilipsHue::CLightState::ALERT::LSELECT;
  light1.Set(state, errors);

  state.m_Alert = PhilipsHue::CLightState::ALERT::NONE;
  state.m_Brightness = 12ui8;
  state.m_TransitionTime = 1ui16;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Alert = PhilipsHue::CLightState::ALERT::NOT_SET;
  state.m_Brightness.reset();
  state.m_TransitionTime.reset();
  state.m_BrightnessInc = 4ui16;
  for (int i{0}; i<100; i++)
  {
    light1.Set(state, errors);
    Sleep(100);
  }

  state.m_BrightnessInc.reset();
  state.m_XY = std::pair<float, float>{0.675f, 0.322f};
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_XY.reset();
  state.m_XYInc = std::pair<float, float>{-0.01f, 0.01f};
  for (int i{0}; i<100; i++)
  {
    light1.Set(state, errors);
    Sleep(100);
  }

  state.m_XYInc.reset();
  state.m_Hue = 12750ui16;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Hue.reset();
  state.m_HueInc = 30ui16;
  for (int i{0}; i<1000; i++)
  {
    light1.Set(state, errors);
    Sleep(100);
  }

  state.m_HueInc.reset();
  state.m_CT = 500ui16;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_CT.reset();
  state.m_CTInc = -3ui16;
  for (int i{0}; i<100; i++)
  {
#pragma warning(suppress: 26486)
    light1.Set(state, errors);
    Sleep(100);
  }

  state.m_CTInc.reset();
  state.m_Effect = PhilipsHue::CLightState::EFFECT::COLORLOOP;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Effect = PhilipsHue::CLightState::EFFECT::NONE;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Effect = PhilipsHue::CLightState::EFFECT::NOT_SET;
  state.m_Saturation = 25ui8;
#pragma warning(suppress: 26486)
  light1.Set(state, errors);

  state.m_Saturation.reset();
  state.m_SaturationInc = 3i16;
  for (int i{0}; i<100; i++)
  {
#pragma warning(suppress: 26486)
    light1.Set(state, errors);
    Sleep(100);
  }

  hr = light1.Refresh(errors);
#pragma warning(suppress: 26477)
  ATLVERIFY(SUCCEEDED(hr));
  bool bOn{false};
  hr = light1.GetOn(bOn);
  if (SUCCEEDED(hr))
    printf("Light 1 On: %d\n", static_cast<int>(bOn));
  uint8_t bri{0};
  hr = light1.GetBrightness(bri);
  if (SUCCEEDED(hr))
    printf("Light 1 brightness: %d\n", static_cast<int>(bri));
  uint16_t hue{0};
  hr = light1.GetHue(hue);
  if (SUCCEEDED(hr))
    printf("Light 1 hue: %d\n", static_cast<int>(hue));
  uint8_t saturation{0};
  hr = light1.GetSaturation(saturation);
  if (SUCCEEDED(hr))
    printf("Light 1 saturation: %d\n", static_cast<int>(saturation));
  uint16_t colorTemp{0};
  hr = light1.GetColorTemperature(colorTemp);
  if (SUCCEEDED(hr))
    printf("Light 1 color temperature: %d\n", static_cast<int>(colorTemp));
  bool bReachable{false};
  hr = light1.GetReachable(bReachable);
  if (SUCCEEDED(hr))
    printf("Light 1 reachable: %d\n", static_cast<int>(bReachable));
  std::wstring sColorMode;
  hr = light1.GetColorMode(sColorMode);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 color mode: %s\n", sColorMode.c_str());
  PhilipsHue::CLightState::ALERT alert{PhilipsHue::CLightState::ALERT::NOT_SET};
  hr = light1.GetAlert(alert);
  if (SUCCEEDED(hr))
    printf("Light 1 alert: %d\n", static_cast<int>(alert));
  PhilipsHue::CLightState::EFFECT effect{PhilipsHue::CLightState::EFFECT::NOT_SET};
  hr = light1.GetEffect(effect);
  if (SUCCEEDED(hr))
    printf("Light 1 effect: %d\n", static_cast<int>(effect));
  xy = std::pair<float, float>{0.0f, 0.0f};
  hr = light1.GetXY(xy);
  if (SUCCEEDED(hr))
    printf("Light 1 xy: %f, %f\n", static_cast<double>(xy.first), static_cast<double>(xy.second));
  std::wstring sType;
  hr = light1.GetType(sType);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 type: %s\n", sType.c_str());
  std::wstring sModelID;
  hr = light1.GetModelID(sModelID);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 modelid: %s\n", sModelID.c_str());
  std::wstring sName;
  hr = light1.GetName(sName);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 name: %s\n", sName.c_str());
  std::wstring sUniqueID;
  hr = light1.GetUniqueID(sUniqueID);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 uniqueid: %s\n", sUniqueID.c_str());
  std::wstring sManufacturerName;
  hr = light1.GetManufacturerName(sManufacturerName);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 Manufacturer name: %s\n", sManufacturerName.c_str());
  std::wstring sLuminaireUniqueID;
  hr = light1.GetLuminaireUniqueID(sLuminaireUniqueID);
  if (SUCCEEDED(hr))
    wprintf(L"Light 1 Luminaire uniqueid: %s\n", sLuminaireUniqueID.c_str());

  /*
  light1.Refresh(errors);
  light1.Rename("Back Bedroom light One", errors);
  light1.Refresh(errors);
  light1.Rename("Back Bedroom light", errors);
  light1.Refresh(errors);
  light1.Delete(errors);
  */

  for (uint8_t i{1}; i<255; i++)
  {
    PhilipsHue::CLightState state2;
    state2.m_XY = PhilipsHue::RGBToXY(RGB(255, 0, 0));
    state2.m_Brightness = i;
    state2.m_TransitionTime = 1ui16;
    state2.m_On = true;
#pragma warning(suppress: 26486)
    light1.Set(state2, errors);
    Sleep(100);
  }

  for (uint8_t i{1}; i<255; i++)
  {
    PhilipsHue::CLightState state2;
    state2.m_XY = PhilipsHue::RGBToXY(RGB(0, 255, 0));
    state2.m_Brightness = i;
    state2.m_TransitionTime = 1ui16;
    state2.m_On = true;
#pragma warning(suppress: 26486)
    light1.Set(state2, errors);
    Sleep(100);
  }

  for (uint8_t i{1}; i<255; i++)
  {
    PhilipsHue::CLightState state2;
    state2.m_XY = PhilipsHue::RGBToXY(RGB(0, 0, 255));
    state2.m_Brightness = i;
    state2.m_TransitionTime = 1ui16;
    state2.m_On = true;
#pragma warning(suppress: 26486)
    light1.Set(state2, errors);
    Sleep(100);
  }

  hr = light1.SetBrightness(1ui8, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetBrightness(255ui8, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetHue(1000ui16, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetHue(30000ui16, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetSaturation(1000ui8, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetSaturation(30000ui8, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetColorTemperature(500ui16, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetColorTemperature(153ui16, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetAlert(PhilipsHue::CLightState::ALERT::LSELECT, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(5000);
  hr = light1.SetAlert(PhilipsHue::CLightState::ALERT::NONE, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetEffect(PhilipsHue::CLightState::EFFECT::COLORLOOP, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(10000);
  hr = light1.SetEffect(PhilipsHue::CLightState::EFFECT::NONE, errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetXY(std::pair<float, float>(0.4f, 0.35f), errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);
  hr = light1.SetXY(std::pair<float, float>(0.1f, 0.8f), errors); //NOLINT(clang-analyzer-deadcode.DeadStores)
  Sleep(1000);

  //Close down sockets
  WSACleanup();
}
